#  Copyright (c) 2008 patrick coleman

import copy

import Point3

class BoundingBox(object):
	def __init__(self, minPoint, maxPoint):
		self.minPoint = Point3.Point3(minPoint.x, minPoint.y, minPoint.z)
		self.maxPoint = Point3.Point3(maxPoint.x, maxPoint.y, maxPoint.z)
	
	def addPoint(self, point):
		if point.x < self.minPoint.x:
			self.minPoint.x = point.x
		if point.y < self.minPoint.y:
			self.minPoint.y = point.y
		if point.z < self.minPoint.z:
			self.minPoint.z = point.z
			
		if point.x > self.maxPoint.x:
			self.maxPoint.x = point.x
		if point.y > self.maxPoint.y:
			self.maxPoint.y = point.y
		if point.z > self.maxPoint.z:
			self.maxPoint.z = point.z

	def diagonalSquared(self):
		return self.minPoint.distanceSquared(self.maxPoint)

	def center(self):
		return Point3.Point3(0.5 * (self.minPoint.x + self.maxPoint.x),
							 0.5 * (self.minPoint.y + self.maxPoint.y),
							 0.5 * (self.minPoint.z + self.maxPoint.z))
	
	def corners(self):
		res = []
		res.append(copy.copy(self.minPoint))
		res.append(Point3.Point3(self.minPoint.x, self.minPoint.y, self.maxPoint.z))
		res.append(Point3.Point3(self.minPoint.x, self.maxPoint.y, self.minPoint.z))
		res.append(Point3.Point3(self.minPoint.x, self.maxPoint.y, self.maxPoint.z))
		res.append(Point3.Point3(self.maxPoint.x, self.minPoint.y, self.minPoint.z))
		res.append(Point3.Point3(self.maxPoint.x, self.minPoint.y, self.maxPoint.z))
		res.append(Point3.Point3(self.maxPoint.x, self.maxPoint.y, self.minPoint.z))
		res.append(copy.copy(self.maxPoint))
		return res
		
	def closestCorner(self, point):
		closestDist = point.distanceSquared(self.minPoint)
		closest = copy.copy(self.minPoint)
		dist2 = point.distanceSquared(self.maxPoint)
		dist3 = point.distanceSquared(Point3.Point3(self.minPoint.x, self.minPoint.y, self.maxPoint.z))
		dist4 = point.distanceSquared(Point3.Point3(self.minPoint.x, self.maxPoint.y, self.minPoint.z))
		dist5 = point.distanceSquared(Point3.Point3(self.minPoint.x, self.maxPoint.y, self.maxPoint.z))
		dist6 = point.distanceSquared(Point3.Point3(self.maxPoint.x, self.minPoint.y, self.minPoint.z))
		dist7 = point.distanceSquared(Point3.Point3(self.maxPoint.x, self.minPoint.y, self.maxPoint.z))
		dist8 = point.distanceSquared(Point3.Point3(self.maxPoint.x, self.maxPoint.y, self.minPoint.z))
		
		if dist2 < closestDist:
			closestDist = dist2
			closest = copy.copy(self.maxPoint)
		if dist3 < closestDist:
			closestDist = dist3
			closest = Point3.Point3(self.minPoint.x, self.minPoint.y, self.maxPoint.z)
		if dist4 < closestDist:
			closestDist = dist4
			closest = Point3.Point3(self.minPoint.x, self.maxPoint.y, self.minPoint.z)
		if dist5 < closestDist:
			closestDist = dist5
			closest = Point3.Point3(self.minPoint.x, self.maxPoint.y, self.maxPoint.z)
		if dist6 < closestDist:
			closestDist = dist6
			closest = Point3.Point3(self.maxPoint.x, self.minPoint.y, self.minPoint.z)
		if dist7 < closestDist:
			closestDist = dist7
			closest = Point3.Point3(self.maxPoint.x, self.minPoint.y, self.maxPoint.z)
		if dist8 < closestDist:
			closestDist = dist8
			closest = Point3.Point3(self.maxPoint.x, self.maxPoint.y, self.minPoint.z)
		
		return closest

def testAddPoint():
	p = Point3.Point3(1, 1, 1)
	bb = BoundingBox(p, p)
	
	print 'init:     ', bb.minPoint, bb.maxPoint 
	
	bb.addPoint(Point3.Point3(2, 1, 1))
	print 'greater x:', bb.minPoint, bb.maxPoint 
	bb.addPoint(Point3.Point3(1, 3, 1))
	print 'greater y:', bb.minPoint, bb.maxPoint 
	bb.addPoint(Point3.Point3(1, 1, 4))
	print 'greater z:', bb.minPoint, bb.maxPoint 
	
	bb.addPoint(Point3.Point3(-2, 1, 1))
	print 'lesser  x:', bb.minPoint, bb.maxPoint 
	bb.addPoint(Point3.Point3(1, -3, 1))
	print 'lesser  y:', bb.minPoint, bb.maxPoint 
	bb.addPoint(Point3.Point3(1, 1, -4))
	print 'lesser  z:', bb.minPoint, bb.maxPoint 
	
	print bb.diagonalSquared(), bb.center()

def testCorners():
	bb = BoundingBox(Point3.Point3(-1, -1, -1), Point3.Point3(1, 1, 1))
	cs = bb.corners()
	for c in cs:
		print c

def testClosest():
	bb = BoundingBox(Point3.Point3(-1, -1, -1), Point3.Point3(1, 1, 1))
	xs = [-2, 2]
	ys = [-2, 2]
	zs = [-2, 2]
	
	for x in xs:
		for y in ys:
			for z in zs:
				p = Point3.Point3(x, y, z)
				print p, ':', bb.closestCorner(p) 

if __name__ == '__main__':
	testCorners()
	