#  Copyright (c) 2008 patrick coleman

import math
from mmath.Vector2 import Vector2
import Exceptions

class Point2(object):
	"Cartesian 2D Points"
	
	def __init__(self, x = 0, y = 0):
		self.x = x
		self.y = y
		
	def __sub__(self, other):
		if type(other) in [Vector2]:
			return Point2(self.x - other.x, self.y - other.y)
		else:
			return Vector2(self.x - other.x, self.y - other.y)
	
	def __add__(self, other):
		" Vector addition "
		if type(other) in [Vector2]:
			return Point2(self.x + other.x, self.y + other.y)
	
	def __mul__(self, other):
		"scalar mult for barycentric combinations"
		if (type(other) == int or type(other) == float):
			return Point2(self.x * other, self.y * other)
		
	def __rmul__(self, other):
		"scalar mult for barycentric combinations"
		if (type(other) == int or type(other) == float):
			return Point2(self.x * other, self.y * other)
	
	def __div__(self, other):
		"scalar post division for barycentric combinations"
		if (type(other) == int or type(other) == float):
			denom = 1.0 / float(other)
			return self * denom
	
	def __getitem__(self, i):
		if i == 0:
			return self.x
		elif i == 1:
			return self.y
	
	def interpolate(self, t, other):
		return (1.0 - t) * self + t * other
	
	def distance(self, other):
		return (other - self).length()
	
	def distanceSquared(self, other):
		return (other - self).lengthSquared()
			
	def __str__(self):
		return '<<%s, %s>>' % (self.x, self.y)

	def __repr__(self):
		return 'geom.Point2.Point2(%s, %s)' % (self.x, self.y)