#  Copyright (c) 2008 patrick coleman

import math
from mmath.Vector3 import Vector3
from Normal3 import Normal3
import Exceptions

class Point3(object):
	"Cartesian 3D Points"
	
	def __init__(self, x = 0, y = 0, z = 0):
		self.x = x
		self.y = y
		self.z = z
		
	def __sub__(self, other):
		if type(other) in [Vector3, Normal3]:
			return Point3(self.x - other.x, self.y - other.y, self.z - other.z)
		else:
			return Vector3(self.x - other.x, self.y - other.y, self.z - other.z)
	
	def __add__(self, other):
		" Vector addition & point addition for barycentric combinations; returns point3"
		if type(other) in [Vector3, Point3, Normal3]:
			return Point3(self.x + other.x, self.y + other.y, self.z + other.z)
	
	def __mul__(self, other):
		"scalar mult for barycentric combinations"
		if (type(other) == int or type(other) == float):
			return Point3(self.x * other, self.y * other, self.z * other)
		
	def __rmul__(self, other):
		"scalar mult for barycentric combinations"
		if (type(other) == int or type(other) == float):
			return Point3(self.x * other, self.y * other, self.z * other)
	
	def __div__(self, other):
		"scalar post division for barycentric combinations"
		if (type(other) == int or type(other) == float):
			denom = 1.0 / float(other)
			return self * denom
	
	def __getitem__(self, i):
		if i == 0:
			return self.x
		elif i == 1:
			return self.y
		elif i == 2:
			return self.z
	
	def interpolate(self, t, other):
		return (1.0 - t) * self + t * other
	
	def distance(self, other):
		return (other - self).length()
	
	def distanceSquared(self, other):
		return (other - self).lengthSquared()
			
	def __str__(self):
		return '<<%s, %s, %s>>' % (self.x, self.y, self.z)

	def __repr__(self):
		return 'geom.Point3.Point3(%s, %s, %s)' % (self.x, self.y, self.z)