#  Copyright (c) 2008 patrick coleman

import Exceptions
import Point3

import mmath.Matrix4
import mmath.Vector3
import mmath.Vector4

class Transformation(object):
	"A general class for linear transformations"
	
	def __init__(self, matrix):
		if type(matrix) != mmath.Matrix4.Matrix4:
			raise Exceptions.BadInitializationData()
		self.matrixData = matrix
	
	def matrix(self):
		"returns matrix rep.  overide in subclasses if lazy computation is more efficient"
		return self.matrixData
		
	def __call__(self, arg):
		"""Apply this transform to a Point3 or a Vector3.  Default implementation uses the 
		result of self.matrix()"""
		v = []
		t = type(arg)
		if t == Point3.Point3:
			v = mmath.Vector4.Vector4([arg.x, arg.y, arg.z, 1.0])
		elif t == mmath.Vector3.Vector3:
			v = mmath.Vector4.Vector4([arg.x, arg.y, arg.z, 0.0])
		else:
			raise Exceptions.BadArgumentData()
		
		v = self.matrix() * v
		if t == Point3.Point3:
			invW = 1.0 / v[3]
			return Point3.Point3(invW * v[0], invW * v[1], invW * v[2])
		elif t == mmath.Vector3.Vector3:
			return mmath.Vector3.Vector3(v[0], v[1], v[2])
	