#  Copyright (c) 2008 patrick coleman

import Exceptions
import Vector4
import numpy
import numpy.linalg

class Matrix4(object):
	def __init__(self, 
		data = [1.0, 0.0, 0.0, 0.0, 
				0.0, 1.0, 0.0, 0.0, 
				0.0, 0.0, 1.0, 0.0, 
				0.0, 0.0, 0.0, 1.0]):
		if len(data) == 16:
			self.data = [data[0:4], data[4:8], data[8:12], data[12:16]]
			return
		elif len(data) == 4:
			if (len(data[0]) == 4 and 
				len(data[1]) == 4 and
				len(data[2]) == 4 and 
				len(data[3]) == 4):
				
				self.data = data
				return
		raise Exceptions.BadInitializationData()
	
	def __getitem__(self, i):
		"Access to a row, returns list"
		return self.data[i]
	
	def __setitem__(self, i, data):
		"Assignment of an entire row"
		if len(data) == 4:
			self.data[i] = data
			return
		raise Exceptions.BadArgumentData()
	
	def __mul__(self, other):
		t = type(other)
		if t == float or t == int:
			m2 = Matrix4()
			for i in range(4):
				for j in range(4):
					m2[i][j]  = self.data[i][j] * other
			return m2
		elif t == Vector4.Vector4:
			return Vector4.Vector4([
				Vector4.Vector4(self.data[0]) * other,
				Vector4.Vector4(self.data[1]) * other,
				Vector4.Vector4(self.data[2]) * other,
				Vector4.Vector4(self.data[3]) * other])
		elif t == Matrix4:
			m = Matrix4()
			for i in range(4):
				for j in range(4):
					m.data[i][j] = (
						self.data[i][0] * other[0][j] + 
						self.data[i][1] * other[1][j] + 
						self.data[i][2] * other[2][j] + 
						self.data[i][3] * other[3][j])
			return m
		raise Exceptions.BadArgumentData()
	
	def __sub__(self, other):
		if type(other) == Matrix4:
			m = Matrix4()
			for i in range(4):
				for j in range(4):
					m.data[i][j] = self.data[i][j] - other.data[i][j]
			return m
		raise Exceptions.BadArgumentData()
		
	def array(self):
		return self.data[0] + self.data[1] + self.data[2] + self.data[3]
	
	def transpose(self):
		m = Matrix4()
		for i in range(4):
			for j in range(4):
				m.data[i][j] = self.data[j][i]
		return m
	
	def inverse(self):
		a = numpy.array(self.data)
		inva = numpy.linalg.inv(a)
		m = Matrix4()
		for i in range(4):
			for j in range(4):
				m.data[i][j] = inva[i][j]
		return m
		
	def trace(self):
		return self.data[0][0] + self.data[1][1] + self.data[2][2] + self.data[3][3]
		