#  Copyright (c) 2008 patrick coleman

import math

import Exceptions
import Vector3

class Quaternion(object):
	def __init__(self, s, v):
		if type(v) != Vector3.Vector3:
			print type(v)
			raise Exceptions.BadInitializationData()
		self.s = s
		self.v = v
	
	def __add__(self, other):
		if type(other) != Quaternion:
			raise Exceptions.BadArgumentData()
		return Quaternion(self.s + other.s, self.v + other.v)
	
	def __mul__(self, other):
		"Quaternion or scalar postmultiplication"
		if type(other) not in [Quaternion, float, int]:
			raise Exceptions.BadArgumentData()
		
		# Beware low precedence ordering on ^ operator...
		if type(other) == Quaternion:
			return Quaternion(
				self.s * other.s - self.v * other.v, 
				self.s * other.v + other.s * self.v + (self.v ^ other.v))
		else:
			return Quaternion(other * self.s, other * self.v)
			
	def __rmul__(self, other):
		"Scalar premultiplication"
		if type(other) not in [int, float]:
			raise Exceptions.BadArgumentData()
		return Quaternion(other * self.s, other * self.v)
	
	def __neg__(self):
		return Quaternion(-self.s, -self.v)
	
	def dot(self, other):
		"4D Dot product"
		return self.s * other.s + self.v * other.v
	
	def inverse(self):
		"Conjugate"
		scale = 1.0 / (self.s * self.s + self.v.x * self.v.x + self.v.y * self.v.y + self.v.z * self.v.z)
		return Quaternion(scale * self.s, - scale * self.v) 
		
	def magnitude(self):
		return math.sqrt(self.s * self.s + self.v.x * self.v.x + self.v.y * self.v.y + self.v.z * self.v.z)
	
	def normalize(self):
		scale = 1.0 / self.magnitude()
		return Quaternion(scale * self.s, scale * self.v)
	
	def identity():
		return Quaternion(1, Vector3.Vector3(0.0, 0.0, 0.0))
	identity = staticmethod(identity)
	
	def __str__(self):
		return '[' + str(self.s) + ', ' + str(self.v) + ']'
	
	def __repr__(self):
		return 'mmath.Quaternion.Quaternion(' + str(self.s) + ', ' + repr(self.v) + ')'
		
	def __getitem__(self, i):
		if i == 0:
			return self.s
		else:
			return self.v[i - 1]
	
	def __setitem__(self, i, value):
		if i == 0:
			self.s = value
		else:
			self.v[i - 1] = value
	