#  Copyright (c) 2008 patrick coleman

"Class(es) for Vectors"

import math

class Vector2(object):
	"Cartesian 3D Vectors"
	
	def __init__(self, x = 0, y = 0):
		self.x = x
		self.y = y
	
	def __sub__(self, other):
		return Vector2(self.x - other.x, self.y - other.y)
	
	def __add__(self, other):
		return Vector2(self.x + other.x, self.y + other.y)
	
	def __rmul__(self, other):
		if (type(other) == int or type(other) == float):
			return Vector2(self.x * other, self.y * other)
	
	def __mul__(self, other):
		"Dot product and scalar mult"
		if (type(other) == int or type(other) == float):
			return Vector2(self.x * other, self.y * other)
		elif type(other) == Vector2:
			return self.x * other.x + self.y * other.y
	
	def __neg__(self):
		return Vector2(- self.x, - self.y)
	
	def __div__(self, other):
		"Scalar division"
		invDenom = 1.0 / other
		return invDenom * self
	
	def length(self):
		return math.sqrt(self * self)
	
	def lengthSquared(self):
		return self * self
	
	def normalize(self):
		invLen = 1.0 / self.length()
		return invLen * self
		
	def angle(self, other):
		"returns the angle between two vectors, in radians"
		if type(other) == Vector2:
			return math.acos((self * other) / (self.length() * other.length()))
	
	def interpolate(self, t, other):
		return (1.0 - t) * self + t * other
		
	def __getitem__(self, i):
		if i == 0:
			return self.x
		elif i == 1:
			return self.y
			
	def __str__(self):
		return '<%s, %s>' % (self.x, self.y)

	def __repr__(self):
		return 'mmath.Vector2.Vector2(%s, %s)' % (self.x, self.y)
		