#  Copyright (c) 2008 patrick coleman

"Class(es) for Vectors"

import math

class Vector3(object):
	"Cartesian 3D Vectors"
	
	def __init__(self, x = 0, y = 0, z = 0):
		self.x = x
		self.y = y
		self.z = z
	
	def __sub__(self, other):
		return Vector3(self.x - other.x, self.y - other.y, self.z - other.z)
	
	def __add__(self, other):
		return Vector3(self.x + other.x, self.y + other.y, self.z + other.z)
	
	def __rmul__(self, other):
		if (type(other) == int or type(other) == float):
			return Vector3(self.x * other, self.y * other, self.z * other)
	
	def __mul__(self, other):
		"Dot product and scalar mult"
		if (type(other) == int or type(other) == float):
			return Vector3(self.x * other, self.y * other, self.z * other)
		elif type(other) == Vector3:
			return self.x * other.x + self.y * other.y + self.z * other.z
	
	def __xor__(self, other):
		"Cross product"
		if type(other) == Vector3:
			return Vector3(
				self.y * other.z - self.z * other.y,
				self.z * other.x - self.x * other.z,
				self.x * other.y - self.y * other.x)
	
	def __neg__(self):
		return Vector3(- self.x, - self.y, - self.z)
	
	def __div__(self, other):
		"Scalar division"
		invDenom = 1.0 / other
		return invDenom * self
	
	def length(self):
		return math.sqrt(self * self)
	
	def lengthSquared(self):
		return self * self
	
	def normalize(self):
		invLen = 1.0 / self.length()
		return invLen * self
		
	def angle(self, other):
		"returns the angle between two vectors, in radians"
		if type(other) == Vector3:
			arg = (self * other) / (self.length() * other.length())
			
			# numerical error way past python's print limit can make this slightly over 1.0 which can make
			# acos return nan.  this was a fun bug to find
			if arg > 1.0:
				arg = 1.0
			
			return math.acos(arg)
	
	def interpolate(self, t, other):
		return (1.0 - t) * self + t * other
		
	def __getitem__(self, i):
		if i == 0:
			return self.x
		elif i == 1:
			return self.y
		elif i == 2:
			return self.z
			
	def __str__(self):
		return '<%s, %s, %s>' % (self.x, self.y, self.z)

	def __repr__(self):
		return 'mmath.Vector3.Vector3(%s, %s, %s)' % (self.x, self.y, self.z)
		
def testAngleBug():
	meanVec1 = Vector3(-3.4315922401946392e-16, -0.79335328710012143, 0.60876149832872362)
	meanVec2 = Vector3(-3.1098183297451634e-16, -0.79335328710012298, 0.60876149832872151)
	x =  (meanVec1 * meanVec2) / (meanVec1.length() * meanVec2.length())
	print repr(x)
	print math.acos((meanVec1 * meanVec2) / (meanVec1.length() * meanVec2.length()))
	print meanVec1.angle(meanVec2)

if __name__ == '__main__':
	testAngleBug()