#  Copyright (c) 2008 patrick coleman

import maya.cmds as mc
import mmaya.Node
import mmath.Vector3
import random
import geom.Rotations
import math

class BadSelection(Exception): pass

def makeShader(nameBase, color = None):
	if color == None:
		color = (random.uniform(0.0, 1.0), random.uniform(0.0, 1.0), random.uniform(0.0, 1.0))
	
	lambertNode = mc.shadingNode('lambert', asShader = True, name = nameBase + 'Lambert')
	mc.setAttr(lambertNode + '.color', color[0], color[1], color[2], type = 'double3')
	lambertSG = mc.sets(renderable = True, 
						noSurfaceShader = True, 
						empty = True, 
						name = lambertNode + 'SG')
	mc.connectAttr(lambertNode + '.outColor', lambertSG + '.surfaceShader', force = True)
	return lambertSG
	
def makePrim(node, kind = 'sphere'):
	fullPathTokens = node.name.split('|')
	prim = None
	scale = 1.0
	if kind == 'sphere':
		prim = mc.sphere(name = fullPathTokens[-1] + 'Prim')
		scale = 0.5
	else:
		prim = mc.polyCube(name = fullPathTokens[-1] + 'Prim')
	return prim[0], scale

def geomacizeTree(node, primKind = 'box', ar = 1.0, thicknessLimit = 2.0, shader = None):
	kids = node.children()
	for k in kids:
		# assumes constant kid translation
		translate = mc.getAttr(k.name + '.translate')[0]
		translate = mmath.Vector3.Vector3(translate[0], translate[1], translate[2]) 
		tLen = translate.length()
		if tLen > 0.0:
			prim, scale = makePrim(node, primKind)
			prim = mc.parent(prim, node.name, relative = True)[0]
			if shader != None:
				mc.sets(prim, forceElement = shader)
			mc.setAttr(prim + '.translate', 0.5 * translate.x, 0.5 * translate.y, 0.5 * translate.z) 
			thickness = min(thicknessLimit, scale * tLen * ar)
			mc.setAttr(prim + '.scale', thickness, scale * tLen, thickness) 
			
			# get rotation that aligns prim to child translation
			up = mmath.Vector3.Vector3(0, 1, 0)
			axis = up ^ translate
			angle = up.angle(translate)
			if axis.length() > 0.001 and angle > 0.001:
				qRot = geom.Rotations.QRotation(axis.normalize(), angle)
				eRot = qRot.eulerRotation(local = False)
				rx = math.degrees(eRot.rx)
				ry = math.degrees(eRot.ry)
				rz = math.degrees(eRot.rz)
				mc.setAttr(prim + '.rotate', rx, ry, rz)
		geomacizeTree(k, primKind, ar, thicknessLimit, shader)


def CreateSkeletonGeometry(primKind = 'sphere', ar = 0.25, thicknessLimit = 3.0):
	# test selection
	sel = mc.ls(selection = True)
	if sel is None:
		raise BadSelection()
	if len(sel) < 1:
		raise BadSelection()

	for s in sel:
		nt = mc.nodeType(s)
		if nt not in ['transform', 'joint']:
			raise BadSelection()

	#call recursive function on root

	shader = makeShader(sel[0])
	for s in sel:
		geomacizeTree(mmaya.Node.Node(s), 
				  primKind = primKind, 
				  ar = ar, 
				  thicknessLimit = thicknessLimit,
				  shader = shader)

	mc.select(sel)
	