#  Copyright (c) 2008 patrick coleman

import mmaya.Streamline
import motion.Streamline
import geom.BoundingBox
import ohlone.Maya

class PositionConstraint(object):
	def __init__(self, startTime, endTime, position, offsets = {}):
		self.startTime = startTime
		self.endTime = endTime
		self.position = position
		self.offsets = offsets
	
	def __repr__(self):
		return ('mmaya.MotionConstraints.PositionConstraint(' + 
			str(self.startTime) + ', ' + str(self.endTime) + ', ' + 
			repr(self.position) + ', '+ repr(self.offsets) + ')')
	
	def __cmp__(self, other):
		return cmp(self.startTime, other.startTime)
		
class ConstraintDetector(object):
	def __init__(self, joint, minSamples, noiseTol, streamline = None):
		self.joint = joint
		self.minSamples = minSamples
		self.twoNoiseTol2 = 4.0 * noiseTol * noiseTol
		self.streamline = streamline
		
		self._constraints = None
		self._computeConstraints()
		
	
	def constraints(self):
		return self._constraints

	def _computeConstraints(self):
		# check streamline and build if none passed in
		if self.streamline == None:
			# get a streamline
			positionEvaluator = mmaya.Streamline.SpaceEvaluator(self.joint, space = 'world')
			slCreator = motion.Streamline.StreamlineCreator(int(ohlone.Maya.sceneStartTime()),
															int(ohlone.Maya.sceneEndTime()),
															positionEvaluator)
			self.streamline = slCreator.streamline()
		
		# detect constraints
		self._constraints = []
		searchIndex = 0
		while searchIndex < len(self.streamline):
			res = self._constraintDetect(self.streamline, searchIndex, 
										 self.minSamples, self.twoNoiseTol2)
			if res == None:
				searchIndex += 1
			else:
				self._constraints.append(res[0])
				searchIndex = res[1]	
	
	def _constraintDetect(self, sl, i, minSamples, twoNoiseTol2):
		if len(sl) - i < minSamples:
			return None
		
		bb = geom.BoundingBox.BoundingBox(sl.points[i], sl.points[i])
		j = i + 1
		while j - i < minSamples:
			bb.addPoint(sl.points[j])
			j += 1
		
		# failure test in min # of samples
		if bb.diagonalSquared() > twoNoiseTol2:
			return None
		
		# ok constraint, set to those found so far
		startTime = sl.times[i]
		endTime = sl.times[j - 1]
		position = bb.center()
		
		# add points and update constraint params until j == len(sl) or bb.diagonal > tol
		ended = False
		while j < len(sl) and not ended:
			bb.addPoint(sl.points[j])
			j += 1
			if bb.diagonalSquared() <= twoNoiseTol2:
				endTime = sl.times[j - 1]
				position = bb.center()
			else:
				ended = True
		
		# return constarint and next index to consider constraints from
		return (PositionConstraint(startTime, endTime, position), j) 

if __name__ == '__main__':
	pc = PositionConstraint(1, 10, 19)
	print pc.startTime, pc.endTime, pc.position