#  Copyright (c) 2008 patrick coleman

import maya.cmds as mc

class Node(object):
	"""Proxy class for getting information about maya nodes without
	needing to write contorted commands with cryptic arguments"""
	
	def __init__(self, name):
		if not mc.objExists(name):
			raise Exceptions.NoMayaNode()
		self.name = name
	
	def parentName(self):
		"Returns name of parent or None if top-level node"
		parent = mc.listRelatives(self.name, parent = True)
		if parent == None:
			return None
		else:
			return parent[0]
		
	def parent(self):
		parName = self.parentName()
		if parName == None:
			return None
		else:
			return Node(parName)
	
	def depth(self):
		"Returns depth (of DAG nodes only!) in hierarchy"
		node = self
		depth = -1
		while node != None:
			node = node.parent()
			depth += 1
		
		return depth
	
	def children(self, allNodes = False):
		"""Returns transform and joint children by default, all children if allNodes = True
		Empty list if none"""
		
		allKids = mc.listRelatives(self.name, c = True)
		if allKids == None:
			return []
		if allNodes:
			return [Node(kid) for kid in allKids]
		transformsAndJoints = []
		for kid in allKids:
			nt = mc.nodeType(kid)
			if nt == 'transform' or nt == 'joint':
				transformsAndJoints.append(Node(kid))
		return transformsAndJoints
		
	def descendants(self, allNodes = False):
		
		kids = self.children(allNodes = allNodes)
		desc = []
		for k in kids:
			kDesc = k.descendants(allNodes = allNodes)
			for kd in kDesc:
				desc.append(kd)
		return (kids + desc)