import maya.cmds as mc
import mmath.Vector3
import geom.Point3
import mmaya

def plot2dVectorField(f, xa = -10.0, xb = 10.0, 
				ya = -10.0, yb = 10.0, 
				xStep = 1.0, yStep = 1.0, z = 0.0,
				length = 0.5, name = 'field'):
	'''
	plots a 2d vector field in maya
	f should evaluate a Vector2 over (x, y)
	length should be about 0.5 * min(xStep, yStep)
	'''
	
	arrowT = 0.75
	arrowWidth = 0.15 * length
	
	# maya lines
	lines = []
	
	x = xa
	
	while x <= xb:
		y = ya
		while y <= yb:
			v = f(x, y)
			if v.length() > 0.0:
				v = v.normalize()
			v = length * v
			v3d = mmath.Vector3.Vector3(v.x, v.y, 0.0)
			perpV3d = mmath.Vector3.Vector3(v.y, -v.x, 0.0)
			here = geom.Point3.Point3(x, y, z)
			startPt = here - 0.5 * v3d
			endPt = here + 0.5 * v3d
			arrowStart1 = startPt + arrowT * (endPt - startPt) + arrowWidth * perpV3d
			arrowStart2 = startPt + arrowT * (endPt - startPt) - arrowWidth * perpV3d
			crv = mmaya.makeCurve([startPt, endPt], name + 'Line#')
			lines.append(crv)
			arrowHead = mmaya.makeCurve([arrowStart1, endPt, arrowStart2], name + 'ArrowHead#')
			lines.append(arrowHead)
			
			
			y += yStep
		x += xStep
	
	mc.group(lines, name = name)