#  Copyright (c) 2008 patrick coleman

import motion.Streamframe
import maya.cmds as mc
from mmath.Matrix4 import Matrix4
from mmath.Vector4 import Vector4
from mmath.Vector3 import Vector3
from geom.Point3 import Point3

class MRotatingFrameEvaluator(motion.Streamframe.FrameEvaluator):
	def __init__(self, 
				 transform, 
				 localFrame = motion.Streamframe.Frame(Point3(1, 0, 0), 
													   Point3(0, 1, 0), 
													   Point3(0, 0, 1)), # what's rotated 
				 normalize = False, 
				 radius = 1.0):
		motion.Streamframe.FrameEvaluator.__init__(self)
		self.transform = transform
		self.localFrame = localFrame
		self.normalize = normalize
		self.radius = radius
		
		self.fVectorX = Vector4([localFrame.axisX.x, localFrame.axisX.y, localFrame.axisX.z, 1.0])
		self.fVectorY = Vector4([localFrame.axisY.x, localFrame.axisY.y, localFrame.axisY.z, 1.0])
		self.fVectorZ = Vector4([localFrame.axisZ.x, localFrame.axisZ.y, localFrame.axisZ.z, 1.0])
	
	def frame(self, time):
		" transform self.frame based on transformation at time t, using only rotation"
		
		# get maya rotation matrix
		m = mc.getAttr(self.transform + '.m', t = time)
		mMatrix = Matrix4(m)
		mRotateMatrix = mMatrix.transpose()
		mRotateMatrix[0][3] = 0
		mRotateMatrix[1][3] = 0
		mRotateMatrix[2][3] = 0
		
		# get rotated frame
		rfX = mRotateMatrix * self.fVectorX
		rfY = mRotateMatrix * self.fVectorY
		rfZ = mRotateMatrix * self.fVectorZ
		
		# optional normalization & return frame
		if self.normalize:
			vList = [Vector3(rfX[0], rfX[1], rfX[2]),
					 Vector3(rfY[0], rfY[1], rfY[2]),
					 Vector3(rfZ[0], rfZ[1], rfZ[2])]
			for i in range(3):
				if vList[i].lengthSquared() > 0.0:
					vList[i] = vList[i].normalize()
					vList[i] = self.radius * vList[i]
			return motion.Streamframe.Frame(Point3(vList[0].x, vList[0].y, vList[0].z),
											Point3(vList[1].x, vList[1].y, vList[1].z),
											Point3(vList[2].x, vList[2].y, vList[2].z))
		else:
			return motion.Streamframe.Frame(Point3(rfX[0], rfX[1], rfX[2]),
											Point3(rfY[0], rfY[1], rfY[2]),
											Point3(rfZ[0], rfZ[1], rfZ[2]))

		
		