#  Copyright (c) 2008 patrick coleman

from motion.StreamlineCollections import *
from geom.Point3 import Point3
import maya.cmds as mc
from mmaya.Node import *
import motion.Streamline
import mmaya.Streamline

class ChildSetCreator(StreamlineSetCreator):
	def __init__(self, motion, attribute, defaultPoint = Point3(0, 0, -1)):
		StreamlineSetCreator.__init__(self, motion, attribute)
		self.defaultPoint = defaultPoint
	
	def streamlines(self):
		if self.attribute.split('.')[1] == 'rotate':
			sls = []
			
			# get kids of node of attribute
			transform = self.attribute.split('.')[0]
			kids = Node(transform).children()
			kids = [k.name for k in kids]
			
			for i in range(len(kids)):
				childEvaluator = mmaya.Streamline.ChildEvaluator(transform, i)
				sc = motion.Streamline.StreamlineCreator(self.motion.startTime,
														 self.motion.endTime,
														 childEvaluator)
				sl = sc.streamline()
				sls.append(sl)
		
			return sls
		else:
			raise UnsupportedData()

class TranslationSetCreator(StreamlineSetCreator):
	"creates a set of a single streamlin that moves the pivot around by the translation value"
	def __init__(self, motion, attribute):
		StreamlineSetCreator.__init__(self, motion, attribute)
	
	def streamlines(self):
		node, attr = self.attribute.split('.')
		if attr == 'translate':
			sls = []
			pList = mc.xform(node, rp = True, q = True)
			p = Point3(pList[0], pList[1], pList[2])
			
			# TODO change this to use a TranslationEvaluator
			translationEvaluator = mmaya.Streamline.TranslationEvaluator(node, p)
			sc = motion.Streamline.StreamlineCreator(self.motion.startTime,
													 self.motion.endTime,
													 translationEvaluator)
			sl = sc.streamline()
			sls.append(sl)
		
			return sls
		else:
			raise UnsupportedData()