#  Copyright (c) 2008 patrick coleman

from Exceptions import *

import motion.Motions as Motions

class MotionFilter(object):
	def __init__(self, motion, sampleRate = 1):
		self.motion = motion
		self.sampleRate = sampleRate
	
	def output(self):
		raise AbstractMethod()

class SmoothingFilter(MotionFilter):
	def __init__(self, motion, iterations = 1, sampleRate = 1):
		MotionFilter.__init__(self, motion, sampleRate)
		self.iterations = iterations
	
	def output(self):
		m = self.motion
		
		for i in range(self.iterations):
			sm = Motions.SampledMotion(m.name + "Temp", 
									   m.startTime,
									   m.endTime,
								       m.attributes,
								   	   self.sampleRate)
		
			numSamples = sm.numSamples()
			sampleTimes = [sm.sampleTime(i) for i in range(numSamples)]
		
			for attr in m.attributes:
				sampleValues = [m.attributeValue(attr, st) for st in sampleTimes]
			
				sm.appendSample(attr, sampleValues[0])
		
				for i in range(1, numSamples - 1):
					sm.appendSample(attr, 
									0.25 * sampleValues[i - 1] +
									0.5 * sampleValues[i] +
									0.25 * sampleValues[i + 1])
			
				sm.appendSample(attr, sampleValues[numSamples - 1])
			m = sm
		
		m.name = self.motion.name + 'Smoothed'
		return m
			
		