#  Copyright (c) 2008 patrick coleman

import random

from Exceptions import *

import Streamframe

class StreamframeFilter(object):
	def __init__(self, streamframe):
		self.streamframe = streamframe
	
	def output(self):
		raise AbstractMethod()
	
class SmoothingFilter(StreamframeFilter):
	def __init__(self, streamframe, bias = 0.5, iterations = 1):
		StreamframeFilter.__init__(self, streamframe)
		self.bias = bias
		self.iterations = iterations
	
	def output(self):
		last = self.streamframe
		if self.iterations == 0:
			return Streamframe.Streamframe(last.frames[:], last.times)
		
		for i in range(self.iterations):
			smoothed = Streamframe.Streamframe(last.frames[:], last.times)
			for j in range(1, len(last) - 1):
				smoothedAX = (self.bias * last.frames[j].axisX + 
					0.5 * (1.0 - self.bias) * (last.frames[j - 1].axisX + last.frames[j + 1].axisX))
				smoothedAY = (self.bias * last.frames[j].axisY + 
					0.5 * (1.0 - self.bias) * (last.frames[j - 1].axisY + last.frames[j + 1].axisY))
				smoothedAZ = (self.bias * last.frames[j].axisZ + 
					0.5 * (1.0 - self.bias) * (last.frames[j - 1].axisZ + last.frames[j + 1].axisZ))
				smoothed.frames[j] = Streamframe.Frame(smoothedAX, smoothedAY, smoothedAZ)
			last = smoothed
			
		return last
	