#  Copyright (c) 2008 patrick coleman

from Exceptions import *

import motion.StreamlineFilters

class StreamlineAnalyzer(object):
	def __init__(self, streamline):
		self.streamline = streamline
	
	def poseSignal(self):
		"""Returns a continuous signal, representing a belief about posiness for the streamline"""
		raise AbstractMethod()
		
	def eventSequence(self):
		"""Returns a list of events representing significant things that happen"""
		raise AbstractMethod()

class KinkAnalyzer(StreamlineAnalyzer):
	"""Does not currently support event sequences"""

	def __init__(self, streamline, smoothingBias = 0.5, smoothingIterations = 50):
		StreamlineAnalyzer.__init__(self, streamline)
		filter = motion.StreamlineFilters.SmoothingFilter(self.streamline, 
														  smoothingBias, 
														  smoothingIterations)
		self.smoothedSl = filter.output()
		
	def poseSignal(self):
		"""This class returns the kink measurement of the streamline"""
		signal = []
		for i in range(len(self.smoothedSl)):
			k = self.smoothedSl.kink(i)
			signal.append(k)
		return signal


