#  Copyright (c) 2008 patrick coleman

"""A StreamlineSet is a set of streamlines.  A StreamlineDictionary is a dictionary of attribute-to-streamline"""

from Exceptions import *
from mrandom import sphrand

import Streamline

class StreamlineSetCreator(object):
	def __init__(self, motion, attribute):
		self.motion = motion
		self.attribute = attribute

	def sets(self):
		"Returns a list of streamlines"
		raise AbstractMethod()

class SamplingSetCreator(StreamlineSetCreator):
	"Creates a set of streamlines by sampling points on S2"
	def __init__(self, motion, attribute, numPoints = 10, radius = 1.0):
		StreamlineSetCreator.__init__(self, motion, attribute)
		self.numPoints = numPoints
		self.radius = radius
	
	def streamlines(self):
		"Randomized.  Call random.seed to repeat results"
		if self.attribute.split('.')[1] == 'rotate':
			sls = []
			for i in range(self.numPoints):
				p = sphrand(self.radius)
				slEvaluator = Streamline.RotatingPointEvaluator(self.motion,
					self.attribute,
					point = p,
					degrees = True)
				slCreator = Streamline.StreamlineCreator(self.motion.startTime,
					self.motion.endTime,
					slEvaluator)
				sls.append(slCreator.streamline())
			return sls
		else:
			raise UnsupportedData()

class StreamlineDictionaryCreator(object):
	"currently doesn't support translation attrs"
	def __init__(self, motion, setCreatorClass, setCreatorClassArgs, analysisAttribute = 'rotate'):
		self.motion = motion
		self.setCreatorClass = setCreatorClass
		self.setCreatorClassArgs = setCreatorClassArgs
		self.analysisAttribute = analysisAttribute
	
	def streamlineDictionary(self):
		dict = {}
		for attr in self.motion.attributes:
			if attr.split('.')[1] == self.analysisAttribute:
				pArgs = (self.motion, attr)
				setCreator = apply(self.setCreatorClass, pArgs, self.setCreatorClassArgs)
				dict[attr] = setCreator.streamlines()
		return dict
				

