#  Copyright (c) 2008 patrick coleman

import math

class Extremum(object):
	def __init__(self, index, value):
		self.index = index
		self.value = value

	def __repr__(self):
		return ('msignal.' + str(self.__class__.__name__) + '(' + 
			str(self.index) + ', ' + str(self.value) + ')')

class Maximum(Extremum):
	def __cmp__(self, other):
		"Highest-value max will be considered 'less' for priority in sorting"
		return cmp(other.value, self.value)

class Minimum(Extremum):
	def __cmp__(self, other):
		"Lowwest-value min will be considered 'less' for priority in sorting"
		return cmp(self.value, other.value)

def localMaxima(signal, returnType = 'Index'):
	maxima = []
	for i in range(1, len(signal) - 1):
		if (signal[i] > signal[i - 1] and signal[i] > signal[i + 1]):
			if returnType == 'Object':
				maxima.append(Maximum(i, signal[i]))
			else:
				maxima.append(i)
	return maxima
	
def semilocalMaxima(signal, width = 5, returnType = 'Index'):
	"""
		Locates "semi-local" maxima, where the maxima is a local maxima within
		some window of samples.  Set width to drive this (window width expands to center
		 the sample if width is even)
	"""
	maxima = []
	halfWidth = width / 2
	
	for i in range(1, len(signal) - 1):
		
		# get search bounds
		startIndex = i - halfWidth
		endIndex = i + halfWidth
		if startIndex < 0:
			startIndex = 0
		if endIndex > (len(signal) - 1):
			endIndex = len(signal) - 1
		
		# determine maximality
		isMaxima = True
		for j in range(startIndex, endIndex + 1):
			if signal[i] <= signal[j] and i != j:
				isMaxima = False
				break
		
		# append maxma if found 
		if isMaxima:
			if returnType == 'Object':
				maxima.append(Maximum(i, signal[i]))
			else:
				maxima.append(i)
	
	return maxima

def localMinima(signal, returnType = 'Index'):
	minima = []
	for i in range(1, len(signal) - 1):
		if (signal[i] < signal[i - 1] and signal[i] < signal[i + 1]):
			if returnType == 'Object':
				minima.append(Minimum(i, signal[i]))
			else:
				minima.append(i)
	return minima

def semilocalMinima(signal, width = 5, returnType = 'Index'):
	"""
		Locates "semi-local" minima, where the min is a local min within
		some window of samples.  Set width to drive this (window width expands to center
		 the sample if width is even)
	"""
	minima = []
	halfWidth = width / 2
	
	for i in range(1, len(signal) - 1):
		
		# get search bounds
		startIndex = i - halfWidth
		endIndex = i + halfWidth
		if startIndex < 0:
			startIndex = 0
		if endIndex > (len(signal) - 1):
			endIndex = len(signal) - 1
		
		# determine maximality
		isMinimum = True
		for j in range(startIndex, endIndex + 1):
			if signal[i] >= signal[j] and i != j:
				isMinimum = False
				break
		
		# append maxma if found 
		if isMinimum:
			if returnType == 'Object':
				minima.append(Minimum(i, signal[i]))
			else:
				minima.append(i)
	
	return minima

if __name__ == '__main__':
	numSamples = 20
	signal = [math.sin(i) for i in xrange(numSamples)]
	maxima = localMaxima(signal, returnType = 'Object')
	minima = localMinima(signal, returnType = 'Object')
	maxima.sort()
	minima.sort()
	
	print 'signal'
	for i in xrange(len(signal)):
		print '\t' + str(i) + ', ' + str(signal[i])
	print 'maxima:'
	for m in maxima:
		print '\t' + str(m)
	print 'minima:'
	for m in minima:
		print '\t' + str(m)
		