#  Copyright (c) 2008 patrick coleman

from maya import cmds as mc
import Exceptions

def transformAttributes(node):
	return [node + '.translate', node + '.rotate']
	
def hierarchyTransformAttributes(node):
	nodeType = mc.nodeType(node)
	if (nodeType == 'joint' or nodeType == 'transform'):
		attrs = transformAttributes(node)
		children = mc.listRelatives(node, children = True)
		if children != None:
			for c in children:
				attrs = attrs + hierarchyTransformAttributes(c)
		return attrs
	return []
	
def componentAttributes(attr):
	return [attr + suffix for suffix in ['X', 'Y', 'Z']]
	
def isDestination(attr):
	return mc.connectionInfo(attr, isDestination = True)

def sourceNode(attr):
	if not isDestination(attr): return None
	srcAttr = mc.connectionInfo(attr, sfd = True)
	nodename, attrname = srcAttr.split('.')
	return nodename

def sceneStartTime():
	return mc.playbackOptions(query = True, minTime = True)
	
def sceneEndTime():
	return mc.playbackOptions(query = True, maxTime = True)
	
def currentTime():
	return mc.currentTime(query = True)

def setCurrentTime(time):
	mc.currentTime(time)

def selection():
	"Returns list of selected object names"
	sel = mc.ls(sl = True)
	if sel == None:
		return []
	return sel
	

		
		
		
		
		